@@ -1,10 +1,14 @@
 #-*-python-*-
 from __future__ import print_function
 
+import sys
+try:
+    sys.path.remove('/Applications/Regina.app/Contents/MacOS/python')
+except ValueError:
+    pass
 import regina
 
 import snappy
-import sys
 import time
 import itertools
 
@@ -22,13 +26,13 @@ from tnorm.norm_ball import *
 from tnorm.sage_types import *
 from tnorm.utilities import *
 from tnorm.x3d_to_html import *
-
 import tnorm.constants
 
+
+
 #from sympy import Matrix
 #from sympy import Rational as QQ
 
-QUIET = tnorm.constants.QUIET
 
 #preparser(False)
 
@@ -49,19 +53,29 @@ class TN_wrapper():
 	sage: W = TN_wrapper(string)       # where string is the triangulation encoded as a string (i.e., contents of a tri file)
 	
 	"""
-	def __init__(self, manifold, qtons=None, tracker=False, quiet=False, allows_non_admissible=False, basis='natural', force_simplicial_homology=False):
+	def __init__(self, manifold, qtons=None, quiet=False, tracker=False, allows_non_admissible=False, bdy_H1_basis='natural', force_simplicial_homology=False):
 		
+		self.triangulation = None
 		if isinstance(manifold,str):
-			self.manifold = snappy.Manifold(manifold).filled_triangulation()
+			self.manifold = snappy.Manifold(manifold)
 		elif isinstance(manifold, regina.engine.SnapPeaTriangulation):
 			self.manifold = snappy.Manifold(manifold.snapPea())
+			self.triangulation = manifold
 		elif isinstance(manifold, regina.engine.Triangulation3):
 			self.manifold = snappy.Manifold(manifold.snapPea())
+			self.triangulation = manifold
 		elif isinstance(manifold, snappy.Manifold):
-			self.manifold = manifold.filled_triangulation()
+			self.manifold = manifold
 		elif isinstance(manifold, snappy.Triangulation):
 			self.manifold = manifold
 
+		for c in self.manifold.cusp_info():
+			if c.is_complete == False:
+				self.manifold = self.manifold.filled_triangulation()
+				break
+		
+		self._QUIET = quiet
+		self._force_simplicial_homology = force_simplicial_homology
 		self.num_cusps = self.manifold.num_cusps()
 		if self.num_cusps != 0:
 			try: 
@@ -69,77 +83,105 @@ class TN_wrapper():
 				self.knows_link_complement = True
 			except ValueError:
 				self.knows_link_complement = False
-			if self.knows_link_complement and basis == 'natural':
-				self.H1_basis = 'natural'
+			if self.knows_link_complement and bdy_H1_basis == 'natural':
+				self.bdy_H1_basis = 'natural'
 			else:
 				self.manifold.set_peripheral_curves('shortest')
-				self.H1_basis = 'shortest'
+				self.bdy_H1_basis = 'shortest'
 
 			self.triangulation = regina.SnapPeaTriangulation(self.manifold._to_string())
 			self._angle_structure = solve_lin_gluingEq(self.triangulation)
 			self._intersection_matrices = [intersection_mat(self.manifold, self.triangulation, cusp) for cusp in range(self.triangulation.countCusps())]
 			self.manifold_is_closed = False
 		else:
-			self.triangulation = regina.Triangulation3(self.manifold._to_string())
+			if self.triangulation == None:
+				self.triangulation = regina.Triangulation3(self.manifold._to_string())
+				for _ in range(10):
+					self.triangulation.intelligentSimplify()
 			self.manifold_is_closed = True
-
-
+			self.bdy_H1_basis = None
+		if not self.triangulation.isOriented():
+			self.triangulation.orient()
 		self._qtons = qtons
 		self._tkr = False
 		self.allows_non_admissible = allows_non_admissible
 		self.is_fibered = 'unknown'
-		self.betti = self.triangulation.homologyH1().rank()
-		QUIET = quiet
+		self.betti_number = self.triangulation.homologyH1().rank()
 
 		# qtons memoization caches-----
 		self._euler_char = {}
 		self._map_to_ball = {}
 		self._map_to_H2 = {}
 		self._num_boundary_comps = {}
-		self._over_face = {}
+		self._over_facet = {}
 		self._is_norm_minimizing = {}
 		self._is_admissible = {}
 		self._qtons_image_in_C2 = {}
+
 		if self.manifold.num_cusps() > 0:
 			self._map_to_H1bdy = {}
 			self._regina_bdy_slopes = {}
-		if not QUIET:		
+
+		if not self._QUIET:		
 			print('Enumerating quad transversely oriented normal surfaces (qtons)... ', end='')
-			sys.stdout.flush()
+			try:
+				sys.stdout.flush()
+			except AttributeError:
+				pass
+
 		if tracker:
 			self._tkr = regina.ProgressTracker()
+
+		# compute transversely oriented normal surfaces
 		self._qtons = self.qtons()
+
+		# name the surfaces by their index in the NormalSurfaces list
 		for i in range(self._qtons.size()):
 			self._qtons.surface(i).setName(str(i))
-		if not QUIET:
-			print('Done.')
 
-		if self.manifold.homology().betti_number() > self.manifold.num_cusps():
+		if not self._QUIET:
+			print('Done.')
+			try:
+				sys.stdout.flush()
+			except AttributeError:
+				pass
+
+		if self.betti_number > self.num_cusps:
 			self.has_internal_homology = True
 		else:
 			self.has_internal_homology = False
 
 		# if the manifold has internal homology or force_simplicial_homology==True then we need to use simplicial homology.
-		if self.has_internal_homology or force_simplicial_homology:
+		if self.has_internal_homology or self._force_simplicial_homology:
 			
 			self.uses_simplicial_homology = True
-			if not QUIET:
-				print('computing simplicial homology...')
-				sys.stdout.flush()
+
+			if not self._QUIET:
+				print('computing simplicial homology...',end='')
+				try:
+					sys.stdout.flush()
+				except AttributeError:
+					pass
 
 			self._face_map_to_C2 = get_face_map_to_C2(self.triangulation)
 			self._quad_map_to_C2 = get_quad_map_to_C2(self.triangulation, self._face_map_to_C2)
 			H2_basis_in_C2, P, qtons_image = H2_as_subspace_of_C2(self, self._face_map_to_C2, self._quad_map_to_C2)
 			self._project_to_im_del3 = P
 			self._qtons_image_in_C2 = {i:qtons_image[i] for i in range(len(qtons_image))}
-			assert len(H2_basis_in_C2) == self.betti
-			I = Matrix.identity(self.betti)
+			assert len(H2_basis_in_C2) == self.betti_number, self.manifold.name()+', force_simplicial_homology={}'.format(self.force_simplicial_homology)
+				# raise an error if the qtons do not generate H2. This should only happen for a knot in a 
+				# rational homology sphere, with force_simplicial_homology=True.
+			I = Matrix.identity(self.betti_number)
 			B = Matrix(H2_basis_in_C2).transpose()
 			A = B.solve_left(I)
 			self._map_H2_to_standard_basis = A
-			if not QUIET:
+
+			if not self._QUIET:
 				print('Done.')
-				sys.stdout.flush()
+				try:
+					sys.stdout.flush()
+				except AttributeError:
+					pass
 		else:
 			self.uses_simplicial_homology = False
 
@@ -159,8 +201,6 @@ class TN_wrapper():
 		else:
 			return None
 
-
-
 	def qtons(self):
 		"""
 		Enumerate vertex oriented quad normal surfaces for the ideal triangulation self.triangulation()
@@ -186,11 +226,15 @@ class TN_wrapper():
 
 	def show_progress(self):
 		print(self._tkr.percent())
+		try:
+			sys.stdout.flush()
+		except AttributeError:
+			pass
 
 
 	def euler_char(self,qtons):
 		"""
-		Return the Euler characteristic of the spun normal surface. 
+		Return the Euler characteristic of the normal surface. 
 		Argument can be a surface (Regina oriented spun normal surface) or the index of a surface.
 
 		sage: W.euler_char(3)    # return Euler characteristic of the surface with index 3, i.e, W.OrientedNormalSurfaceList.surface(3).
@@ -215,8 +259,12 @@ class TN_wrapper():
 
 	def boundary_slopes(self,qtons,quad_mat=None):
 		"""
-		Return the image of the given surface in H1(\partial M). Argument can be a surface (regina oriented
-		spun normal surface) or the index of a surface.
+		Return the boundary slope of the given surface in H1(\partial M), with respect to the basis W.bdy_H1_basis. 
+		Argument can be a surface (regina quad transverely oriented normal surface) or the index of a surface.
+
+		Note that this slope may not be the same as W.regina_boundary_slopes(surface), as Regina computes slopes based on
+		an orientation that depends on the direction that the spun normal surface spins into the cusp. Our orientation on 
+		boundary slopes is instead based on the transverse orientation of the surface.
 
 		sage: W._map_to_H1bdy(3)    # return image in H1(\partial M) of the surface with index 3, i.e, W.OrientedNormalSurfaceList.surface(3).
 		[(-1, 0), (0, 1), (-1, 0)]
@@ -238,6 +286,10 @@ class TN_wrapper():
 				return h1
 
 	def regina_bdy_slopes(self,qtons,quad_mat=None):
+		""" Return boundary slopes that result if we ignore transverse orientation, and cancel slopes that spin into the cusp
+		in opposite directions. This may differ from slopes as computed by Regina by a sign, but should otherwise be the same.
+		"""
+
 		if self.manifold_is_closed:
 			return []
 		else:
@@ -256,8 +308,8 @@ class TN_wrapper():
 
 	def map_to_H2(self,qtons):
 		"""
-		Return the image of the given surface in H2(M, \partial M). Argument can be a surface (regina oriented
-		spun normal surface) or the index of a surface.
+		Return the image of the given surface in H2(M, \partial M). Argument can be a surface (regina quad transversely oriented
+		normal surface) or the index of a surface.
 
 		sage: W.mapToH2(3)    # return the image in H2(M,\partial M) of the surface with index 3, i.e, W.OrientedNormalSurfaceList.surface(3).
 		(0,1,0)
@@ -281,6 +333,10 @@ class TN_wrapper():
 			return self._simplicial_map_to_H2(qtons)
 
 	def simplicial_class(self,qtons):
+		""" Return the vector in the dimension 2 chain group C2 corresponding to the qtons surface. The i^th coordinate
+			of this vector corresponds to the number of triangles of index i, as indexed by Regina, with orientation positive
+			if the transverse orientation of the surface points toward the tetrahedron triangle(i).front(). 
+		"""
 		if not self.uses_simplicial_homology:
 			return None
 		else:
@@ -296,26 +352,29 @@ class TN_wrapper():
 				self._qtons_image_in_C2[ind] = c
 			return c
 
-	def over_face(self, qtons, as_string=False):
-
+	def over_facet(self, qtons, as_string=False):
+		"""Returns the facet of the norm ball that the qtons surface is above. E.g., <0 1 5> corresponds to the facet 
+		with vertices 0, 1, and 5. 
+		"""
 		try:
 			ind = int(qtons)
 		except TypeError:
 			ind = int(qtons.name())
 
-		if ind in self._over_face:
-			f = self._over_face[ind]
+		if ind in self._over_facet:
+			f = self._over_facet[ind]
 		else:
 			v = self.map_to_H2(ind)
-			f = over_face_(v,self.norm_ball.polyhedron)
-			self._over_face[ind] = f
+			f = over_facet_(v,self.norm_ball.polyhedron)
+			self._over_facet[ind] = f
 		if as_string:
 			return None if f==None else '<{}>'.format(' '.join([str(v.index()) for v in f.vertices()]))
 		else:
 			return f
 
 	def is_norm_minimizing(self, qtons):
-
+		"""Return True if the given qtons surface is Thurston norm minimizing.
+		"""
 		try:
 			ind = int(qtons)
 		except TypeError:
@@ -332,8 +391,8 @@ class TN_wrapper():
 				self._is_norm_minimizing[ind] = False
 				return False
 
-	def locally_compatible(self,qtons1,qtons2):
-		pass
+#	def locally_compatible(self,qtons1,qtons2):
+#		pass
 
 	def is_admissible(self, qtons):
 		try:
@@ -427,33 +486,6 @@ class TN_wrapper():
 				image.append((i,pt,ec))
 		return image
 
-#	@cached_property
-#	def _norm_ball_points(self):
-#		"""
-#		Return the points in H2(M,bdy M) which are images of (normalized) vertices of the oriented spun
-#		normal surface projective solution space with negative Euler characteristic. The convex hull of 
-#		these points is the norm ball.
-#		"""
-#		points = dict()
-#		rays = dict()  # if the Euler char of s is 0 we can't normalize. This means the norm ball is
-#		               # infinite in the direction of map_to_H2(s), so it contains the ray (0,map_to_H2(s)).
-#		image_w_euler = self._image_in_H2M()
-#
-#		for (i,pt,ec) in image_w_euler:
-#			if ec < 0:
-#				quad_mat = oriented_quads_mat(self.qtons().surface(i))
-#				normalized = tuple([coord/(-ec) for coord in pt])
-#				if normalized not in points:
-#					points[tuple([coord/(-ec) for coord in pt])] = i # divide by euler char to normalize
-#				elif self.boundary_slopes(i,quad_mat) == self.regina_bdy_slopes(i,quad_mat):
-#					points[tuple([coord/(-ec) for coord in pt])] = i
-#				elif abs_slopes(self.boundary_slopes(i)) == abs_slopes(self.regina_bdy_slopes(i)):
-#					if not self.boundary_slopes(points[normalized]) == self.regina_bdy_slopes(points[normalized]):
-#						points[tuple([coord/(-ec) for coord in pt])] = i
-#			elif ec == 0:
-#				rays[tuple(pt)] = i
-#		return points, rays
-
 	@cached_property
 	def _norm_ball_points(self):
 		"""
@@ -462,37 +494,50 @@ class TN_wrapper():
 		these points is the norm ball.
 		"""
 		points0 = dict()
+		points = dict()
 		rays = dict()  # if the Euler char of s is 0 we can't normalize. This means the norm ball is
 		               # infinite in the direction of map_to_H2(s), so it contains the ray (0,map_to_H2(s)).
 		image_w_euler = self._image_in_H2M()
 
+
 		for (i,pt,ec) in image_w_euler:
 			if ec < 0:
 				normalized = tuple([coord/(-ec) for coord in pt]) # divide by euler char to normalize
 
-				if not self.uses_simplicial_homology:
-					if normalized not in points0:
-						points0[normalized] = (i,0) 
-					elif points0[normalized][1] == 2:
-						continue
-						
-					bs = self.boundary_slopes(i)
-					rbs = self.regina_bdy_slopes(i)
-					d = gcd([gcd(s) for s in bs])				
-	
-					if d==1 and bs==rbs:
-						points0[normalized] = (i,2) 
-					elif d==1:
-						points0[normalized] = (i,1) 
+#				if not self.uses_simplicial_homology:
+				if normalized not in points0:
+					points0[normalized] = []
+					points0[normalized].append((i,pt,ec))
 				else:
-					if normalized not in points0:
-						points0[normalized] = (i,0)
-					else:
-						continue
+					points0[normalized].append((i,pt,ec))
 
 			elif ec == 0:
 				rays[tuple(pt)] = i
-		points = dict([(key,points0[key][0]) for key in points0])
+
+		for nzd in points0:
+
+			min_ec = min(-rep[2] for rep in points0[nzd])
+			reps = [rep[0] for rep in points0[nzd] if -rep[2]==min_ec] # restrict to norm minimizing reps
+
+			if self._force_simplicial_homology:
+				# order reps by L^1 norm
+				reps = sorted(reps,key=lambda x: sum(self.simplicial_class(x).apply_map(abs)))
+				points[nzd] = reps[0] # get the index of the rep with minimal L^1 norm
+			else:
+				# order according to how well self.boundary_slopes(i) agrees with self.regina_bdy_slopes(i)
+				if self.num_boundary_comps != 0:
+					def sort_fn(rep):
+						bs = self.boundary_slopes(rep)
+						rbs = self.regina_bdy_slopes(rep)
+						if bs == rbs:
+							return 0
+						elif all([rbs[i] in [bs[i],tuple([-bs[i][j] for j in range(len(bs[i]))])] for i in range(len(rbs))]):
+							return 1
+						else:
+							return 2
+					reps = sorted(reps,key=lambda x: sort_fn(x))
+				points[nzd] = reps[0]
+
 		return points, rays
 
 	@cached_property
@@ -501,21 +546,23 @@ class TN_wrapper():
 		Return the norm ball as a sage Polyhedron. Some things you can do:
 
 		sage: B=W.norm_ball
-		sage: B.faces(2)        # list the 2-dimensional faces
-		sage: B.faces(0)        # list the 0-dimensional faces (vertices)
+		sage: B.facets(2)        # list the 2-dimensional facets
+		sage: B.facets(0)        # list the 0-dimensional facets (vertices)
 
-		sage: F = B.faces(2)
-		sage: F[0].vertices()   #return vertices of the 0 face
+		sage: F = B.facets(2)
+		sage: F[0].vertices()   #return vertices of the 0^th facet
 
 		sage: B.plot()          # plot of B. I imagine this only works if B is 1,2, or 3 dimensional.
 
 		The Polyhedron class has a lot of methods, and I have not explored many of them. There is 
 		probably a lot more that would be useful. Use tab completion to explore!
 		"""
-		if not QUIET:
+		if not self._QUIET:
 			print('Computing Thurston norm unit ball... ', end='')
-			sys.stdout.flush()
-
+			try:
+				sys.stdout.flush()
+			except AttributeError:
+				pass
 		pts_dict, rays_dict = self._norm_ball_points
 
 		### If M is not hyperbolic then rays_dict should be non-empty, and the norm ball should be non-compact. 
@@ -524,7 +571,7 @@ class TN_wrapper():
 		if len(rays_dict) != 0:
 			polyhedron = Polyhedron(vertices=Matrix(pts_dict.keys()), rays=Matrix(rays_dict.keys()), base_ring=QQ, backend='cdd')
 			rays = tuple([(rays_dict[tuple(i*vector(v))],i*vector(v)) for v in polyhedron.lines_list() for i in [1,-1]])
-			Rays = [Ray(i,rays[i][0],rays[i][1],self.num_boundary_comps(rays[i][0]), self.euler_char(rays[i][0]), self.boundary_slopes(rays[i][0]), True, self.is_admissible(rays[i][0]),True) for i in range(len(rays))]
+			Rays = [Ray(i,rays[i][0],rays[i][1],self.num_boundary_comps(rays[i][0]), self.euler_char(rays[i][0]), self.boundary_slopes(rays[i][0]), True, self.is_admissible(rays[i][0]),True, self.simplicial_class(rays[i][0])) for i in range(len(rays))]
 			vertices = tuple([(pts_dict[tuple(v)],vector(v)) for v in polyhedron.vertices_list()])
 			projected_verts = {}
 			Vertices = []
@@ -548,9 +595,9 @@ class TN_wrapper():
 			for i in range(len(polyhedron.vertices_list())):
 				v = tuple(polyhedron.vertices_list()[i])
 				if v in pts_dict:
-					Vertices.append(AdornedVertex(i,pts_dict[v],vector(v),self.num_boundary_comps(pts_dict[v]), self.euler_char(pts_dict[v]), self.boundary_slopes(pts_dict[v]), False, self.is_admissible(pts_dict[v]), True))
+					Vertices.append(AdornedVertex(i,pts_dict[v],vector(v),self.num_boundary_comps(pts_dict[v]), self.euler_char(pts_dict[v]), self.boundary_slopes(pts_dict[v]), False, self.is_admissible(pts_dict[v]), True, W.simplicial_class(pts_dict[v])))
 				else:
-					Vertices.append(AdornedVertex(i, None, vector(v), projected_verts[v][0], projected_verts[v][1], projected_verts[v][2],False, None, True))
+					Vertices.append(AdornedVertex(i, None, vector(v), projected_verts[v][0], projected_verts[v][1], projected_verts[v][2],False, None, True,None))
 
 
 
@@ -564,62 +611,79 @@ class TN_wrapper():
 
 			Rays = []
 		ball = TNormBall(Vertices, Rays, polyhedron)
-		if self.num_cusps == 1 and self.betti == 1:
+		if self.num_cusps == 1 and self.betti_number == 1:
+			M = self.manifold.copy()
+			(p,q) = self.manifold.homological_longitude()
+			elem_divs = [div for div in M.homology().elementary_divisors() if div != 0]
+			M.dehn_fill((p,q))
+			filled_elem_divs = [div for div in M.homology().elementary_divisors() if div != 0]
+			b = 1
+			for div in elem_divs:
+				b *= div
+			for div in filled_elem_divs:
+				b /= div
 			A = self.manifold.alexander_polynomial()
 			A_norm = QQ(A.degree() - 1)
-			print(A_norm)
 			try:
 				v=ball.vertices[0]
+
 			except IndexError:
 				pass
 			if A_norm <= 0 or not A.is_monic():
 				self.is_fibered = False
 				ball.confirmed = True
 
-			elif len(ball.vertices)==0 or A_norm > abs(v.euler_char):
+			elif len(ball.vertices)==0:
 				self.is_fibered = True
 				polyhedron = Polyhedron(vertices=[[A_norm],[-A_norm]], base_ring=QQ)
-				Vertices = [AdornedVertex(0, None, (-1/A_norm,), 1, -A_norm, [(0,1)], False, True, False),AdornedVertex(1, None, (1/A_norm,), 1, -A_norm, [(0,-1)], False, True, False)]
+				Vertices = [AdornedVertex(0, None, (-1/A_norm,), b, -A_norm, [(0,1)], False, True, False,None),AdornedVertex(1, None, (1/A_norm,), b, -A_norm, [(0,-1)], False, True, False, None)]
 				ball = TNormBall(Vertices, Rays, polyhedron)
 				ball.confirmed = True
 			elif A_norm == abs(v.euler_char):
+				assert v.num_boundary_comps == b
 				self.is_fibered = 'unknown'
 				ball.confirmed = True
 			elif A_norm < abs(v.euler_char):
 				M = self.manifold.copy()
-				p,q = v.boundary_slopes[0]
-				d = gcd(p,q)
-				M.dehn_fill((p/d,q/d))
+				M.dehn_fill(M.homological_longitude())
 				Mf = M.filled_triangulation()
 				T = regina.Triangulation3(Mf._to_string())
+				boo = T.intelligentSimplify()
 				ns = regina.NormalSurfaces.enumerate(T,regina.NS_QUAD,regina.NS_VERTEX,regina.NS_ALG_DEFAULT)
-				print(ns.size())
 				non_trivial = [i for i in range(ns.size()) if ns.surface(i).isOrientable()]
-				print(len(non_trivial))
 				if len(non_trivial)>1:
 					non_trivial = [i for i in non_trivial if ns.surface(i).cutAlong().isConnected()]
-					print(len(non_trivial))
 				if len(non_trivial) == 0: ## something is wrong!
-					print('Warning: failed to confirm that norm ball is correct (this can happen for knots).')
+					print('Warning: failed to confirm that norm ball is correct (error: len(non_trivial)==0, culprit:M={}.'.format(self.manifold.name()))
 					ball.confirmed = False
 				elif len(non_trivial) >= 1:
 					genus = min([(2-regina_to_sage_int(ns.surface(i).eulerChar()))/2 for i in non_trivial])
-					print(2*genus - 2 + 1)
-					if 2*genus - 2 + 1 == abs(v.euler_char):
+					if 2*genus - 2 + b == abs(v.euler_char):
 						self.is_fibered = False
 						ball.confirmed = True
-					elif 2*genus - 2 + 1 == A_norm:
+					elif 2*genus - 2 + b == A_norm:
+						multiplier = 1/QQ(abs(v.euler_char)/A_norm)
+						if self.uses_simplicial_homology == True:
+							simplicial_class = multiplier*self.simplicial_class(v.surface_index)
+						else:
+							simplicial_class = None
 						self.is_fibered == True
 						polyhedron = Polyhedron(vertices=[[A_norm],[-A_norm]], base_ring=QQ)
-						Vertices = [AdornedVertex(0, None, (-1/A_norm,), 1, -A_norm, [(0,1)], False, True, False),AdornedVertex(1, None, (1/A_norm,), 1, -A_norm, [(0,-1)], False, True, False)]
+						Vertices = [AdornedVertex(0, None, (-1/A_norm,), b, -A_norm, [(0,1)], False, True, False, simplicial_class),AdornedVertex(1, None, (1/A_norm,), b, -A_norm, [(0,-1)], False, True, False, simplicial_class)]
 						ball = TNormBall(Vertices, Rays, polyhedron)
 						ball.confirmed = True
 					else:
-						print('hello')
-						print('Warning: failed to confirm that norm ball is correct (this can happen for knots).')
+						print('Warning: failed to confirm that norm ball is correct (error: 2g-1!=A_norm or abs(X(S)), culprit:M={}.'.format(self.manifold.name()))
 						ball.confirmed = False
-		if not QUIET:
+			else:
+				# if we are here, then something is wrong, and the following will cause an error to be thrown.
+				assert A_norm <= abs(v.euler_char) # (this should never happen)
+		if not self._QUIET:
 			print('Done.')
+			try:
+				sys.stdout.flush()
+			except AttributeError:
+				pass
 		return ball
 
 	@cached_property
@@ -640,8 +704,12 @@ class TN_wrapper():
 
 	@cached_property
 	def qtons_info(self):
-		if not QUIET:
+		if not self._QUIET:
 			print('Analyzing quad transversely oriented normal surfaces... ', end='')
+			try:
+				sys.stdout.flush()
+			except AttributeError:
+				pass
 		qtons_info_dict = dict([(i,{}) for i in range(self.qtons().size())])
 		for i in range(self.qtons().size()):
 			qtons_info_dict[i]['image_in_H2'] = self.map_to_H2(i)
@@ -650,10 +718,14 @@ class TN_wrapper():
 			qtons_info_dict[i]['num_boundary_comps'] = self.num_boundary_comps(i)
 			qtons_info_dict[i]['genus'] = (2-self.euler_char(i)-self.num_boundary_comps(i))/2
 			qtons_info_dict[i]['is_norm_minimizing'] = self.is_norm_minimizing(i)
-			qtons_info_dict[i]['over_face'] = self.over_face(i,as_string=True)
+			qtons_info_dict[i]['over_facet'] = self.over_facet(i,as_string=True)
 			qtons_info_dict[i]['spinning_slopes'] = self.regina_bdy_slopes(i)
-		if not QUIET:
+		if not self._QUIET:
 			print('Done.')
+			try:
+				sys.stdout.flush()
+			except AttributeError:
+				pass
 		return qtons_info_dict
 
 def check_subfaces(ray,f_poly):
@@ -667,30 +739,30 @@ def check_subfaces(ray,f_poly):
 	return f_poly
 			
 
-def over_face_(v, P):
+def over_facet_(v, P):
 	if v.is_zero():
 		return None
 	else:
 		ray = Polyhedron(rays=[v]) # ray generated by v
-		f_poly = check_subfaces(ray, P) # find the face that v lies over, as a polyhedron
+		f_poly = check_subfaces(ray, P) # find the facet that v lies over, as a polyhedron
 
-		# f_poly is the face we want, but it is in the form of a polyhedron, not a face of P. So we find the face of P with the same vertices, and return that.
+		# f_poly is the facet we want, but it is in the form of a polyhedron, not a facet of P. So we find the facet of P with the same vertices, and return that.
 		for i in range(P.dim()):
 			for f in P.faces(i):
 				if sorted([vert.vector() for vert in f.vertices()]) == sorted([vert.vector() for vert in f_poly.vertices()]):
 					return f
 
-def lies_over_face(v, P, face):
-	# find the lowest dimensional face that v lies over
-	v_face = over_face_(v, P)
-	# check if v_face found above is a subface of face
-	if set(v_face.vertices()).issubset(set(face.vertices())) and set(v_face.lines()).issubset(set(face.lines())):
+def lies_over_facet(v, P, facet):
+	# find the lowest dimensional facet that v lies over
+	v_facet = over_facet_(v, P)
+	# check if v_facet found above is a subfacet of facet
+	if set(v_facet.vertices()).issubset(set(facet.vertices())) and set(v_facet.lines()).issubset(set(facet.lines())):
 		return True
 	return False
 
-def basis_over_face(basis, P, face):  # check if all vectors in basis lie over face.
+def basis_over_facet(basis, P, facet):  # check if all vectors in basis lie over facet.
 	for vec in basis:
-		if not lies_over_face(vec, P, face):
+		if not lies_over_facet(vec, P, facet):
 			return False
 	return True
 
