@@ -1,7 +1,8 @@
 
+import sys
 from tnorm.kernel.matrices import sign_matrix
 
-QUIET = None
+#QUIET = None
 SIGN_MATRIX = sign_matrix()
 ABS_SIGN_MATRIX = sign_matrix().apply_map(abs)
 
@@ -14,4 +15,4 @@ ABS_SIGN_MATRIX = sign_matrix().apply_map(abs)
 #    from sage.all import RR
 #    IN_SAGE = True
 #except:
-#    IN_SAGE = False
\ No newline at end of file
+#    IN_SAGE = False
